(function ($) {
$.Redactor.opts.langs['fr'] = {
    html: 'Code HTML',
    video: 'Vidéo',
    image: 'Image',
    table: 'Tableau',
    link: 'Lien',
    link_insert: 'Insérer un lien ...',
    link_edit: 'Editer le lien',
    unlink: 'Nettoyer le lien',
    formatting: 'Format',
    paragraph: 'Texte normal',
    quote: 'Citation',
    code: 'Code',
    header1: 'Titre 1',
    header2: 'Titre 2',
    header3: 'Titre 3',
    header4: 'Titre 4',
    header5: 'Titre 5',
    bold:  'Gras',
    italic: 'Italique',
    fontcolor: 'Couleur du texte',
    backcolor: 'Couleur du fond de ligne',
    unorderedlist: 'Liste à puce',
    orderedlist: 'Liste numérotée',
    outdent: 'Diminuer l\'alinéa',
    indent: 'Augmenter l\'alinéa',
    redo: 'Refaire',
    undo: 'Défaire',
    cut: 'Couper',
    cancel: 'Annuler',
    insert: 'Insérer',
    save: 'Sauvegarder',
    _delete: 'Supprimer',
    insert_table: 'Insérer le tableau',
    insert_row_above: 'Ajouter une ligne au dessus',
    insert_row_below: 'Ajouter une ligne en dessous',
    insert_column_left: 'Ajouter une colonne à gauche',
    insert_column_right: 'Ajouter une colonne à droite',
    delete_column: 'Supprimer la colonne',
    delete_row: 'Supprimer la ligne',
    delete_table: 'Supprimer le tableau',
    rows: 'Lignes',
    columns: 'Colonnes',
    add_head: 'Ajouter un titre',
    delete_head: 'Supprimer le titre',
    title: 'Titre',
    subtitle: 'Sous-titre',
    image_view: 'Voir l\'image',
    image_position: 'Position de l\'image',
    none: 'aucun',
    left: 'à gauche',
    right: 'à droite',
    image_web_link: 'lien de l\'image',
    text: 'Texte',
    mailto: 'Envoyer à ',
    web: 'URL',
    video_html_code: 'Code HTML de la vidéo',
    file: 'Fichier',
    upload: 'Envoyer',
    download: 'Télécharger',
    choose: 'Choisir',
    or_choose: 'Ou choisissez',
    drop_file_here: 'Glissez-déposez le fichier ici',
    align_left: 'Aligner à gauche',
    align_center: 'Aligner au centre',
    align_right: 'Aligner à droite',
    align_justify: 'Justifier',
    horizontalrule: 'Ligne horizontale',
    fullscreen: 'Plein écran',
    deleted: 'Supprimer',
    anchor: 'Ancre',
    link_new_tab: 'Ouvrir le lien dans un nouvel onglet',
    underline: 'Souligner',
    alignment: 'Alignement',
    filename: 'Nom de fichier (optionnel)',
    edit: 'Modifier',
    center: 'Centrer',
    upload_label: 'Déposez le fichier ici ou ',
    redo: 'Refaire',
    undo: 'Défaire'
};
})( jQuery );